# vi: ts=4 expandtab syntax=python
##############################################################################
# Copyright (c) 2008 IBM Corporation
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Eclipse Public License v1.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v10.html
#
# Contributors:
# Scott Moser (IBM) - initial implementation
# Dave Leskovec (IBM) - getDefaultConfiguration detect ovf:default attribute
##############################################################################

"""
Module for interfacing to an OVF and XML
"""

import os
import sha
import re
from xml.dom import Node

from subprocess import call
from subprocess import Popen
from subprocess import PIPE
from subprocess import STDOUT

def createTextDescriptionOfNodeList(nodeList):
    """
    This function will get information from a list of nodes and return a list
    containing the information found for each node

    @param nodeList: The list of nodes to create the message from.
    @type nodeList: List of DOM nodes

    @return: String containing error message.
    """
    i = 0
    errMsg = ' '
    for node in nodeList:
        info = '\n Node '+str(i)+': '
        try:
            info += getNodes(node,(hasTagName, 'Info'))[0].firstChild.data
        except IndexError:
            try:
                info += (getNodes(node,(hasTagName,'rasd:ElementName'))[0].
                            firstChild.data)
            except IndexError:
                try:
                    info += (getNodes(node,(hasTagName,'vssd:ElementName'))[0]
                                .firstChild.data)
                except IndexError:#we look for attributes to print
                    for key in node.attributes.keys():
                        info += key
                        info += '=' + str(node.attributes[key].value) + ' '
        i += 1
        errMsg += info
    return errMsg

def rmNode(node):
    """
    This function will remove a node and all of its children.

    @param node: DOM Node to be deleted
    @type node: DOM Node
    """
    node.parentNode.removeChild(node)

def rmNodeAttributes(node, attributeList=[], strict=False):
    """
    This function will remove the attributes for a given section.
    Passing False to strict will mean that no errors will be thrown
    if an attribute for the given node is not found.

    @param node: DOM Node where attributes will be deleted
    @type node: DOM Node

    @param strict: if True, checks for all attributes before removing
    @type strict: boolean

    @param attributeList: A list of attributes to be deleted
    @type attributeList: list of Strings

    @raise ValueError: if strict=True and any attribute not present
    """
    for attribute in attributeList:
        if strict:
            if not node.hasAttribute(attribute):
                raise ValueError("The attribute, " + attribute +
                                 ", does not exist.")
        else:
            node.removeAttribute(attribute)

    if strict:
        for attribute in attributeList:
            node.removeAttribute(attribute)

def getDefaultConfiguration(ovfDoc):
    """
    Returns identifier for the default configuration
    """
    # first check if ovf specifies a default configuration
    defaultConfigurationNode = getNodes(ovfDoc, (hasTagName, 'Configuration'),
                                        (hasAttribute, 'ovf:default', "true"))
    if defaultConfigurationNode:
        return defaultConfigurationNode[0].attributes['ovf:id'].value

    # first check if ovf specifies a default configuration
    defaultConfigurationNode = getNodes(ovfDoc, (hasTagName, 'Configuration'))
    if defaultConfigurationNode:
        return defaultConfigurationNode[0].attributes['ovf:id'].value

    return None

def isElement(elem):
    """
    Returns a boolean value, after testing if the DOM Node argument is of
    nodeType ELEMENT_NODE.

    @param elem: node
    @type elem: DOM Node

    @return: truth value
    @rtype: boolean
    """
    if elem.nodeType == Node.ELEMENT_NODE:
        return True
    else:
        return False

def hasChildText(elem, childTagName, value):
    """
    Returns a boolean value, after testing if the DOM Node argument has a
    child element with the given tagName and text.

    @param elem: node
    @type elem: DOM Node

    @param childTagName: child Element tag name
    @type childTagName: String

    @param value: text
    @type value: String

    @return: truth value
    @rtype: boolean
    """
    elemList = elem.getElementsByTagName(childTagName)
    if len(elemList) > 0:
        for child in elemList:
            if child.hasChildNodes():
                if child.firstChild.nodeType == Node.TEXT_NODE:
                    if child.firstChild.data == value:
                        return True
        return False
    else:
        return False

def hasParentAttribute(elem, attrName, value):
    """
    Returns a boolean value, after testing if the parent of the DOM Node
    argument has an attribute with the given value.

    @param elem: node
    @type elem: DOM Node

    @param attrName: parent attribute name
    @type attrName: String

    @param value: parent attribute value
    @type value: String

    @return: truth value
    @rtype: boolean
    """
    if isElement(elem):
        return (elem.parentNode.getAttribute(attrName) == value)
    else:
        return False

def hasAttribute(elem, attrName, value):
    """
    Returns a boolean value, after testing if the DOM Node argument
    has an attribute with the given value.

    @param elem: node
    @type elem: DOM Node

    @param attrName: attribute name
    @type attrName: String

    @param value: attribute value
    @type value: String

    @return: truth value
    @rtype: boolean
    """
    if isElement(elem):
        return (elem.getAttribute(attrName) == value)
    else:
        return False

def hasLocalName(node, value):
    """
    Returns a boolean value, after testing if the DOM Node argument has a
    local name with the given value.

    @param node: node
    @type node: DOM Node

    @param value: local name
    @type value: String

    @return: truth value
    @rtype: boolean
    """
    if isElement(elem):
        if (node.localName == value) or (node.localName == 'ovf:'+value) or \
           (node.localName == 'ovm:'+value):
            return True
        else:
            return False

def hasNodeName(node, value):
    """
    Returns a boolean value, after testing if the DOM Node argument has a
    node name with the given value.

    @param node: node
    @type node: DOM Node

    @param value: node name
    @type value: String

    @return: truth value
    @rtype: boolean
    """
    if isElement(node):
        if (node.nodeName == value) or (node.nodeName == 'ovf:'+value) or \
           (node.nodeName == 'ovm:'+value):
            return True
        else:
            return False

def hasTagName(elem, value):
    """
    Returns a boolean value, after testing if the DOM Node argument has a
    tag name with the given value.

    @param elem: node
    @type elem: DOM Node

    @param value: tag name
    @type value: String

    @return: truth value
    @rtype: boolean
    """
    if isElement(elem):
        if (elem.tagName == value) or (elem.tagName == 'ovf:'+value) or \
           (elem.tagName == 'ovm:'+value):
            return True
    else:
        return False

def getChildNodes(ovfNode, *criteria):
    """
    Returns a list of nodes from an XML DOM Document, that meet a
    variable number of criteria functions.

    All functions must take at least one argument, and that argument,
    the first argument, must be a DOM Node. The function must return a
    boolean value. The signature of the function must be of the form,
    function(node, arg1, arg2...), and return True or False.

    @note: 'criteria' tuples must be of the form, (function, *args)

    @param ovfNode: OVF document or element
    @type ovfNode: DOM Node

    @param criteria: filters for limiting results, processed in order
    @type criteria: variable length list of tuples

    @return: child Nodes that meet criteria
    @rtype: list of DOM Nodes
    """
    ret = []

    for child in ovfNode.childNodes:
        passed = True
        for tup in criteria:
            function = tup[0]
            if not function(child, *tup[1:]):
                passed = False
                break

        if passed:
            ret.append(child)

    return ret

def getNodes(ovfNode, *criteria):
    """
    Returns a list of nodes from an XML DOM Document, that meet a
    variable number of criteria functions. Wrapper to getChildNodes,
    that recursively descends descendents.

    All functions must take at least one argument, and that argument,
    the first argument, must be a DOM Node. The function must return a
    boolean value. The signature of the function must be of the form,
    function(node, arg1, arg2...), and return True or False.

    @note: 'criteria' tuples must be of the form, (function, *args)

    @param ovfNode: OVF document or element
    @type ovfNode: DOM Node

    @param criteria: filters for limiting results, processed in order
    @type criteria: variable length list of tuples

    @return: descendent Nodes that meet criteria
    @rtype: list of DOM Nodes
    """
    ret = getChildNodes(ovfNode, *criteria)
    for child in ovfNode.childNodes:
        if child.hasChildNodes():
            ret.extend(getNodes(child, *criteria))

    return ret

def getElementsByTagName(ovfNode, tagName, *criteria):
    """
    Wrapper to L{getNodes}, that recursively descends descendents with built
    in tagName filter. Returns a list of nodes from an XML DOM Document,
    that also meet a variable number of additional criteria functions.

    All functions must take at least one argument, and that argument,
    the first argument, must be a DOM Node. The function must return a
    boolean value. The signature of the function must be of the form,
    function(node, arg1, arg2...), and return True or False.

    @note: 'criteria' tuples must be of the form, (function, *args)

    @param ovfNode: OVF document or element
    @type ovfNode: DOM Node

    @param tagName: restrict by tagName
    @type tagName: String

    @param criteria: filters for limiting results, processed in order
    @type criteria: variable length list of tuples

    @return: descendent Nodes that meet criteria
    @rtype: list of DOM Nodes
    """
    return getNodes(ovfNode, (hasTagName, tagName), *criteria)

def getElementsById(ovfNode, tagName, identValue):
    """
    Wrapper to L{getElementsByTagName} that takes in an identifier, and
    based on an element's tagName, calls L{getElementsByTagName} with
    appropriate filter.

    @param ovfNode: OVF document or element
    @type ovfNode: DOM Node

    @param tagName: tag name
    @type tagName: String

    @param identValue: value for identifier
    @type identValue: String

    @return: descendent Nodes that meet criteria
    @rtype: list of DOM Nodes
    """
    # if none of the tests below match, 'ovf:id' will be used for id
    attrName = "ovf:id"
    if tagName == "Disk":
        attrName = "ovf:diskId"
    elif tagName == "Property":
        attrName = "ovf:key"
    elif(tagName == "Info" or
         tagName == "Description" or
         tagName == "Label" or
         tagName == "Category" or
         tagName == "Annotation" or
         tagName == "Product" or
         tagName == "Vendor" or
         tagName == "License" or
         tagName == "Msg"):
        attrName = "ovf:msgid"

    return getElementsByTagName(ovfNode, tagName,
                                (hasAttribute, attrName, identValue))

def getContentEntities(ovfNode, ovfId=None, vs=True, vsc=True):
    """
    Returns a list of OVF Entities, optionally restricted by id and
    Entity_Type.

    @param ovfNode: OVF document or element
    @type ovfNode: DOM Node

    @param ovfId: VirtualSystem or VirtualSystemCollection id
    @type ovfId: String

    @param vs: search with tagName VirtualSystem
    @type vs: boolean

    @param vsc: search with tagName VirtualSystemCollection
    @type vsc: boolean

    @return: list of entities
    @rtype: list of DOM nodes
    """
    ents = []
    if vs:
        ents.extend(getElementsByTagName(ovfNode, 'VirtualSystem'))
    if vsc:
        ents.extend(getElementsByTagName(ovfNode, 'VirtualSystemCollection'))

    if ovfId == None:
        return ents
    else:
        return [entity for entity in ents
                if entity.getAttribute('ovf:id') == ovfId]

def getDict(ovfSection, configId=None):
    """
    Returns a dictionary, with keys representing the attributes and the
    children, for an ovf section. Optionally, limits to given configuration.
    L{getNodes} can be used to get a list of sections, each of which
    could be passed in for the argument ovfSectionNode.

    @param ovfSection: OVF section
    @type ovfSection: DOM Node

    @param configId: configuration identifier
    @type configId: String

    @return: attributes and data from section
    @rtype: dictionary

    @note: Returned dict keys,
        - name: OVF section tag name
        - node: section DOM Node
        - attr(optional): attributes of section, only valid for some sections
        - children: list of dictionaries for children, contain same basic
                    structure, as they have name, node, and optional attr
                    keys. They may also have a text key for enclosed data
                    and additional keys for their children.

    @todo: modify to handle and OVF DOM Node
    """
    # Setup dictionary for ovf section
    sectDict = dict(node=ovfSection,
                    name=ovfSection.tagName)

    attributes = getAttributes(ovfSection)
    if attributes != None:
        sectDict['attr'] = attributes

    # Fill in dictionary with information from node's children
    if sectDict['node'].hasChildNodes():
        sectDict['children'] = []
        for child in sectDict['node'].childNodes:
            if(child.nodeType == Node.ELEMENT_NODE and
               (not hasNodeName(child, 'Info'))):

                # Restrict by given configuration
                if(configId == None or
                    child.getAttribute('ovf:configuration') == configId):
                    childDict = dict(node=child,
                                     name=child.tagName)
                    getAttributes(child, childDict)
                    sectDict['children'].append(childDict)

                    # Get info from grandchildren, either text, or
                    # data from offspring.
                    if child.hasChildNodes():
                        data = False
                        for grandchild in child.childNodes:
                            if grandchild.nodeType == Node.ELEMENT_NODE:
                                if grandchild.firstChild:
                                    childDict[grandchild.tagName] = \
                                        grandchild.firstChild.data
                                    data = True

                        if not data:
                            # Child has data i.e. AnnotationSection
                            if not childDict.has_key('text'):
                                childDict['text'] = ''
                            childDict['text'] = (childDict['text'] +
                                                 child.firstChild.data)

    return sectDict

def getAttributes(xmlNode, attributes=None):
    """
    Returns a dictionary of the attributes for a DOM node.

    @param xmlNode: ovf element
    @type xmlNode: DOM Node

    @param attributes: attributes dictionary
    @type attributes: dictionary

    @return: new or modified (passed in) attributes dictionary
    @rtype: dictionary
    """
    if xmlNode.hasAttributes():
        if attributes == None:
            attributes = dict()
        for index in range(xmlNode.attributes.length):
            attrName = xmlNode.attributes.item(index).name
            attrValue = xmlNode.attributes.item(index).value
            attributes[attrName] = attrValue

    return attributes

def ovfString(sectDict):
    """
    Return a dictionary returned from L{getDict} as a string.

    @param sectDict: ovf section dictionary, L{getDict}
    @type sectDict: dictionary

    @return: section dictionary as string
    @rtype: String

    @todo: not machine-readable, e.g. escaped quotes
    """
    # list keys to print
    ret = ''
    printList = sectDict.keys()
    printList.remove('node')

    ret += sectDict['name'] + "\n"
    printList.remove('name')

    if sectDict.has_key('attr'):
        for key in sectDict['attr'].keys():
            ret += key + ":= `" + sectDict['attr'][key] + "`\n"

        printList.remove('attr')

    if sectDict.has_key('children'):
        printList.remove('children')
        for child in sectDict['children']:
            ret += ovfString(child) + "\n"

    for key in printList:
        if type(sectDict[key]) == type(dict()):
            ret += ovfString(sectDict[key]) + "\n"
        else:
            ret += key + ":= `" + sectDict[key] + "`\n"

    return ret

def isVirtualSystem(ovfDoc, ovfId):
    """
    Determines if an id corresponds to an Entity node of
    VirtualSystem_Type in an ovf.

    @param ovfDoc: OVF Document
    @type ovfDoc: DOM Document

    @param ovfId: Entity identifier
    @type ovfId: String

    @return: True or False
    @rtype: boolean
    """
    return (getNodes(ovfDoc,
                    (hasTagName, 'VirtualSystem'),
                    (hasAttribute, 'ovf:id', ovfId)) != [])

def isVirtualSystemCollection(ovfDoc, ovfId):
    """
    Determines if an id corresponds to an Entity node of
    VirtualSystemCollection_Type in an ovf.

    @param ovfDoc: OVF Document
    @type ovfDoc: DOM Document

    @param ovfId: Entity identifier
    @type ovfId: String

    @return: True or False
    @rtype: boolean
    """
    return (getNodes(ovfDoc,
                    (hasTagName, 'VirtualSystemCollection'),
                    (hasAttribute, 'ovf:id', ovfId)) != [])

def isConfiguration(ovfDoc, configId):
    """
    Verifies whether a configuration is defined in the ovf.

    @param ovfDoc: OVF Document
    @type ovfDoc: DOM Document

    @param configId: configuration name
    @type configId: String

    @return: tests if configuration exists
    @rtype: boolean

    @raise ValueError: ConfigId doesn't match any in DeploymentOptions
    @raise NotImplementedError: DeploymentOptions not defined
    """
    deploy = getElementsByTagName(ovfDoc, 'DeploymentOptionSection')
    if deploy != []:
        configList = getDict(deploy[0])['children']
        while configList != []:
            config = configList.pop(0)
            if config['ovf:id'] == configId:
                return True

        return False
    else:
        # may want to throw different error if doesn't exist
        raise NotImplementedError("Ovf.isConfiguration: No configurations found.")

def sha1sumFile(path):
    """
    This will give the sha1sum of a given file in hex.

    if file object is given, it is expected to be at the start of the file
    (no rewind/seek(0) will be performed)

    @param path: path to file or file object
    @type path: String

    @return: sha1sum of filename in hex
    @rtype: String
    """
    if  path.__class__.__name__ != "file" and \
        path.__class__.__name__ != "URLGrabberFileObject":
        fd = open(path, "rb")
    else:
        fd = path

    digested = sha.new()
    while 1:
        buf = fd.read(4096)
        if buf == "":
            break
        digested.update(buf)
    res =  digested.hexdigest()
    if  path.__class__.__name__ != "file" and \
        path.__class__.__name__ != "URLGrabberFileObject":
        fd.close()
    return res

def href2abspath(href, base=None):
    """
    Returns the absolute path when passed an href.

    @param href: an "href" from a .ovf file or elsewhere
    @type href: String

    @param base: search in base directory for file if not absolute path href,
                 if base is a file, will search from dirname(base)
    @type base: String

    @return: absolute path to file (or the href if remote)
    @rtype: String
    """

    cwd = os.getcwd()
    if base == None:
        path = cwd
    else:
        if(os.path.isfile(base)):
            path = os.path.dirname(base)
        else:
            path = base

        # dirname returns "" rather than "." for "filename"
        if path == "":
            path = "."

    filePath = None

    if href.startswith("/"):
        filePath = href
    elif os.path.isfile(path + "/" + href):
        filePath = os.path.abspath(path + "/" + href)
    elif os.path.isfile(cwd + "/" + href):
        filePath = os.path.abspath(cwd + "/" + href)
    elif href.find("://") != -1:
        filePath = href

    if filePath == None:
        # check if there are chunks with href
        # regex to find chunks 
        chunkre = re.compile(os.path.basename(href) + '.' + '[0-9]+')

        dir = os.path.dirname(path + "/" + href)
        if os.path.isdir(dir):
            for f in os.listdir(dir):
                if chunkre.match(f):
                    filePath = os.path.abspath(path + "/" + href)
                    break

    # file(s) doesn't exist as far as we can see

    return(filePath)

def getNumChunks(fileSize, chunkSize):
    chunks = fileSize / chunkSize
    if (fileSize % chunkSize) == 0:
        numChunks = chunks
    else:
        numChunks = chunks + 1

    return numChunks

def getChunkFiles(refFile, dirpath):
    fileSize = int(refFile.size)
    chunkSize = int(refFile.chunksize)
    numChunks = getNumChunks(fileSize, chunkSize)

    bname = (os.path.split(os.path.basename(refFile.href)))[1]

    hrefDirName = os.path.dirname(refFile.href)
    if hrefDirName != '':
        chnkDirPath = dirpath + "/" + hrefDirName
    else:
        chnkDirPath = dirpath 

    chunkFiles = []
    for chnk in range(numChunks):
        chunkname = chnkDirPath + "/" + bname + (".%09d") % (chnk)
        if os.path.isfile(chunkname):
            chunkFiles.append(os.path.basename(chunkname))

    return chunkFiles

def sortByIndex(f1, f2):
    index1 = f1.rfind('.')
    index2 = f2.rfind('.')

    if index1 != -1 and index2 != -1:
        i1 = int(f1[index1+1:len(f1)])
        i2 = int(f2[index2+1:len(f2)])
        if i1 > i2:
            return 1
        elif i1 == i2:
            return 0
        else:
            return -1

def splitFile(file, fileSize, chunkSize, destDir):
    numChunks = getNumChunks(fileSize, chunkSize)

    fname = (os.path.split(os.path.basename(file)))[1]

    try:
        inf = open(file, 'rb')
    except (OSError, IOError), e:
        raise RuntimeError, str(e)

    bytesRead = 0
    for chnk in range(numChunks):
        chunkname = destDir + "/" + fname + (".%09d") % (chnk)

        # for last chunk, calculate the correct chunksize
        if chnk == numChunks-1:
            chunksz = fileSize - bytesRead
        else:
            chunksz = chunkSize

        try:
            data = inf.read(chunksz)
            bytesRead += len(data)
            cf = open(chunkname, 'wb')
            cf.write(data)
            cf.close()
        except (OSError, IOError), e:
            print e
            continue
        except EOFError, e:
            print e
            break

def combineChunkFiles(cf, destDir, targetFile):
    try:
        cmbf = open(targetFile, 'ab')
        bufsz = 10485760 # 10MB
        for f in cf:
            inf = open(destDir + "/" + f, 'rb')
            while True:
                data = inf.read(bufsz)
                cmbf.write(data)
                if len(data) != bufsz:
                    break
            inf.close()
        cmbf.close()
    except (OSError, IOError, EOFError), e:
        raise RuntimeError, str(e)

def getMimeType(filepath):
    pObject = Popen(["/usr/bin/file","-b","--mime",filepath],
                    stdout=PIPE, stderr=STDOUT)
    pStdout = (pObject.communicate()[0])
    if pObject.returncode:
        raise IOError, pStdout

    pStdout = pStdout.strip()
    if (pStdout.endswith('x-gzip')):
        return 'x-gzip'
    elif (pStdout.endswith('x-bzip2')):
        return 'x-bzip2'

def compressFile(filepath, compression):
    dirpath = os.path.dirname(os.path.abspath(filepath))

    if compression == "gzip":
         mimeType = getMimeType(filepath)
         if not mimeType == 'x-gzip':
             # compress the file
             tfile = os.path.join(dirpath, "tmpfile")
             #gzip -c file > tmpfile && mv -f tmpfile file 
             pObject = Popen(["/bin/gzip","-c",filepath], 
                             stdout=file(tfile,"w"))
             pStdout = pObject.communicate()[0]
             if pObject.returncode:
                 raise IOError, pStdout

             pObject = Popen(["/bin/mv","-f",tfile,filepath], 
                             stdout=PIPE, stderr=STDOUT)
             pStdout = pObject.communicate()[0]
             if pObject.returncode:
                 raise IOError, pStdout

    elif compression == "bzip2":
        mimeType = Ovf.getMimeType(filepath)
        if not mimeType == 'x-bzip2':
            # compress the file
            pObject = Popen(["/bin/bzip2","-c",filepath], 
                            stdout=file(tfile,"wb"))
            pStdout = pObject.communicate()[0]
            if pObject.returncode:
                raise IOError, pStdout

            pObject = Popen(["/bin/mv","-f",tfile,filepath], 
                            stdout=PIPE, stderr=STDOUT)
            pStdout = pObject.communicate()[0]
            if pObject.returncode:
                raise IOError, pStdout

def uncompressFile(filepath, compression):
    dirpath = os.path.dirname(os.path.abspath(filepath))

    if compression == "gzip":
         mimeType = getMimeType(filepath)
         if mimeType == 'x-gzip':
             # compress the file
             tfile = os.path.join(dirpath, "tmpfile")
             #gzip -dc file > tmpfile && mv -f tmpfile file 
             pObject = Popen(["/bin/gzip","-dc",filepath], 
                             stdout=file(tfile,"w"))
             pStdout = pObject.communicate()[0]
             if pObject.returncode:
                 raise IOError, pStdout

             pObject = Popen(["/bin/mv","-f",tfile,filepath], 
                             stdout=PIPE, stderr=STDOUT)
             pStdout = pObject.communicate()[0]
             if pObject.returncode:
                 raise IOError, pStdout

    elif compression == "bzip2":
        mimeType = Ovf.getMimeType(filepath)
        if mimeType == 'x-bzip2':
            # compress the file
            pObject = Popen(["/bin/bzip2","-dc",filepath], 
                            stdout=file(tfile,"wb"))
            pStdout = pObject.communicate()[0]
            if pObject.returncode:
                raise IOError, pStdout

            pObject = Popen(["/bin/mv","-f",tfile,filepath], 
                            stdout=PIPE, stderr=STDOUT)
            pStdout = pObject.communicate()[0]
            if pObject.returncode:
                raise IOError, pStdout

def createFile(filename, format, size, populatedSize=None, sparse=True):
    # FIXME: add format, populatedSize and parent support
    dir = os.path.dirname(filename)
    if not os.path.exists(dir):
        os.makedirs(dir)
    fd = None
    try:
        try:
            fd = os.open(filename, os.O_WRONLY | os.O_CREAT | os.O_DSYNC)
            if sparse:
                os.ftruncate(fd, size)
            else:
                bufsize = 1024 * 1024
                for i in range(0, size / bufsize):
                    os.write(fd, '\x00' * bufsize)
                os.write(fd, '\x00' * size % bufsize)
        except OSError, err:
            raise RuntimeError('Error creating file %s: %s' % (filename, err))
    finally:
        if fd is not None:
            os.close(fd)

def convertFile(srcFilename, srcFormat, dstFilename, dstFormat,
                srcCompression=None, srcChunksize=None, dstSize=None,
                srcPopulatedSize=None, dstVariant=None):
    # FIXME: add dstSize, dstPopulatedSize and parent support
    if srcChunksize is not None:
        cmd = ['cat', '%s.*' % srcFilename]
    else:
        cmd = ['cat', srcFilename]
    if srcCompression == "gzip":
        cmd += ['|', 'gzip -dc']
    elif srcCompression == "bzip2":
        cmd += ['|', 'bzip2 -dc']

    if srcFormat == 'vmdk':
        cmd += ['|', '/usr/bin/vbox-img', 'convert', '--stdin', 
                '--srcformat %s' % srcFormat, 
                '--dstfilename %s' % dstFilename, 
                '--dstformat %s' % dstFormat]
        if dstVariant:
            cmd.append('--variant %s' % dstVariant)
    elif srcFormat == 'raw':
        # FIXME: when input is in raw format, only support output to raw.
        cmd += ['|', 'dd', 'of=%s' % dstFilename]
    else:
        raise RuntimeError('Unsupported source file format: %s' % srcFormat)

    cmd_str = ' '.join(cmd)
    proc = Popen(cmd_str, stdout=PIPE, stderr=PIPE,
                  shell=True, close_fds=True)
    (stdoutdata, stderrdata) = proc.communicate()
    if proc.returncode != 0:
        raise RuntimeError('Command: (%s) failed: (%s) %s' % (cmd_str, 
                                                 proc.returncode, stderrdata))

def convertFile2Vmdk(srcFilename, srcFormat, dstFilename, dstFormat,
                     dstVariant=None):
    if srcFormat == 'raw':
        cmd = ['/usr/bin/vbox-img', 'convert', 
                '--srcfilename %s' % srcFilename, 
                '--srcformat %s' % srcFormat, 
                '--dstfilename %s' % dstFilename, 
                '--dstformat %s' % dstFormat]
        if dstVariant:
            cmd.append('--variant %s' % dstVariant)
    else:
        raise RuntimeError('Unsupported source file format: %s' % srcFormat)

    cmd_str = ' '.join(cmd)
    proc = Popen(cmd_str, stdout=PIPE, stderr=STDOUT, shell=True)
    (outdata, errdata) = proc.communicate()
    if proc.returncode != 0:
        raise RuntimeError('Command: (%s) failed: (%s) %s' % (cmd_str, 
                                                 proc.returncode, errdata))

def xmlString(obj, indent="", newline="", encoding='UTF-8'):
    """
    Wrapper function: Returns a String representing the
    DOM object that was passed.

    @param obj: any DOM Node
    @type obj: DOM Object

    @param indent: indentation character
    @type indent: String

    @param newline: newline character
    @type newline: String

    @param encoding: character encoding
    @type encoding: String

    @return: XML document
    @rtype: String
    """
    from StringIO import StringIO

    buf = StringIO()
    xwritexml(obj, buf, indent, indent, newline, encoding)
    return buf.getvalue()

def xwritexml(obj, writer, indent="", addindent="", newl="", encoding=None):
    func = None
    if   obj.nodeType == Node.DOCUMENT_NODE:
        func = document_writexml
    elif obj.nodeType == Node.ELEMENT_NODE :
        func = element_writexml
    elif obj.nodeType == Node.TEXT_NODE    :
        func = text_writexml

    if func != None:
        func(obj, writer, indent, addindent, newl, encoding)
    else:
        obj.writexml(writer, indent, addindent, newl)

def remove_whitespace_nodes(node):
    """
    Removes all whitespace in DOM node.
    http://safari.oreilly.com/0596007973/pythoncook2-CHP-12-SECT-6
    """
    # prepare the list of text nodes to remove (and recurse when needed)
    remove_list = [  ]
    for child in node.childNodes:
        if child.nodeType == Node.TEXT_NODE and not child.data.strip( ):
            # add this text node to the to-be-removed list
            remove_list.append(child)
        elif child.hasChildNodes( ):
            # recurse, it's the simplest way to deal with the subtree
            remove_whitespace_nodes(child)
    # perform the removals
    for node in remove_list:
        node.parentNode.removeChild(node)
        node.unlink( )


## the following is taken and modified /usr/lib/python2.5/xml/dom/minidom.py
## the xwritexml is just a copied writexml function that calls
## the versions copied 'writexml' functions here rather than the
## ones provided by the library.  For nodes other than those needed
## it will call the functions provided by the library
##
## BEGIN COPY / MODIFY from /usr/lib/python2.5/xml/dom/minidom.py

## def _write_data
def _xwrite_data(writer, data):
    "Writes datachars to writer."
    data = data.replace("&", "&amp;").replace("<", "&lt;")
    data = data.replace("\"", "&quot;").replace(">", "&gt;")
    writer.write(data)

## class Element(Node):
## def writexml(self, writer, indent="", addindent="", newl=""):
def element_writexml(self, writer, indent="", addindent="", newl="",
                     encoding=None):
    # indent = current indentation
    # addindent = indentation to add to higher levels
    # newl = newline string
    writer.write(indent + "<" + self.tagName)

    attrs = self._get_attributes()
    a_names = attrs.keys()
    a_names.sort()

    for a_name in a_names:
        writer.write(" %s=\"" % a_name)
        _xwrite_data(writer, attrs[a_name].value)
        writer.write("\"")
    if self.childNodes:
        if(self.childNodes[0].nodeType == Node.TEXT_NODE):
            writer.write(">")
        else:
            writer.write(">%s"%(newl))
        for node in self.childNodes:
            xwritexml(node, writer, indent + addindent, addindent, newl, encoding)
        if(self.childNodes[0].nodeType == Node.TEXT_NODE):
            writer.write("</%s>%s" %(self.tagName, newl))
        else:
            writer.write("%s</%s>%s" % (indent, self.tagName, newl))
    else:
        writer.write("/>%s"%(newl))

## class Document(Node, DocumentLS):
## def writexml(self, writer, indent="", addindent="", newl="",
def document_writexml(self, writer, indent="", addindent="", newl="",
                      encoding=None):
    if encoding is None:
        writer.write('<?xml version="1.0" ?>'+newl)
    else:
        writer.write('<?xml version="1.0" encoding="%s"?>%s' % (encoding, newl))
    for node in self.childNodes:
        xwritexml(node, writer, indent, addindent, newl)


## class Text(CharacterData):
## def writexml(self, writer, indent="", addindent="", newl=""):
def text_writexml(self, writer, indent="", addindent="", newl="", encoding=None):
    _xwrite_data(writer, self.data)

## END COPY / MODIFY from /usr/lib/python2.5/xml/dom/minidom.py
