
import sys
import tarfile
import tempfile

from ovfova import *
from ovf import OvfCertificate

def extract_ovf_from_ova(ova_file, target_dir):    
    """
    Extract OVF envelope file from ova 

    @param ova_file: path to ova file
    @type ova_file: string
    @param target_dir: path to target directory to extract
    @type target_dir: string

    @raise exception:
    """
    if ova_file != None and os.path.isfile(ova_file):
        if tarfile.is_tarfile(ova_file):
            tf = tarfile.open(ova_file, "r")
            mem = tf.next()
            while mem is not None: 
                if mem.name.find("../") != -1 or mem.name.startswith("/"):
                    raise IOError("Unsafe Tar file " + ova_file)
                elif mem.name.endswith(".ovf"):
                    tf.extract(mem, target_dir)
                    break
                mem = tf.next()
        else:
            raise IOError("Specified appliance archive " + ova_file + \
                          " is not a tar file")
            return False
    else:
        raise IOError("Specified appliance archive " + ova_file + \
                      " does not exist") 
        return False

    return True
    
def get_vm_list(ova_file):    
    """
    Get list of Virtual System names from ova

    @param ova_file: path to ova file
    @type ova_file: string

    @raise exception:
    """
    # extract the ovf file to a temporary directory 
    tmpd = tempfile.mkdtemp(dir="/tmp/") 
    extract_ovf_from_ova(ova_file, tmpd)
    fileList = glob.glob(tmpd+'/*.ovf')

    class GetVmListOption:
        ovfFile = fileList[0]

    try:
        vmList = getVmList(GetVmListOption(), args = '')
        shutil.rmtree(tmpd)
        return vmList
    except Exception, e:
        shutil.rmtree(tmpd)
        print e
        return None
    
def extract_vm_from_ova(ova_file, ovf_id, target_dir, virt_sys_type=None, 
                        config_id=None, env_dir=None):    
    """
    Extract the image files for Vitual System with id ovf_id,
    and create the Xm interface config file vm.cfg

    @param ova_file: path to ova file
    @type ova_file: string
    @param ovf_id: id of the Virtual System
    @type ovf_id: string
    @param target_dir: target directory to extract to
    @type target_dir: string
    @param virt_sys_type: virtual system type
    @type virt_sys_type: string
    @param config_id: configuration to use
    @type config_id: string
    @param env_dir: path to environment files
    @type env_dir: string

    @raise exception:
    """
    # extract the ovf file to a temporary directory 
    tmpd = tempfile.mkdtemp(dir="/tmp/") 
    extract_ovf_from_ova(ova_file, tmpd)
    fileList = glob.glob(tmpd+'/*.ovf')

    class CreateXmconfigOption:
        ovfFile = fileList[0]
        ovfId = ovf_id
        ovaFile = ova_file
        targetDir = target_dir
        cfgFile = None
        virtPlatform = virt_sys_type
        configId = config_id
        envDir = env_dir

    try:
        createXmConfigFile(CreateXmconfigOption(), args = '')
        shutil.rmtree(tmpd)
        return True
    except Exception, e:
        shutil.rmtree(tmpd)
        print e
        return False
    
def get_file_list_not_in_ova(ova_file):    
    """
    Get list of Virtual System names from ova

    @param ova_file: path to ova file
    @type ova_file: string

    @raise exception:
    """
    # extract the ovf file to a temporary directory 
    tmpd = tempfile.mkdtemp(dir="/tmp/") 
    extract_ovf_from_ova(ova_file, tmpd)
    fileList = glob.glob(tmpd+'/*.ovf')

    class GetFilesNotInOvaOption:
        ovfFile = fileList[0]

    try:
        fileList = getFilesNotInOva(GetFilesNotInOvaOption(), args = '')
        shutil.rmtree(tmpd)
        return fileList
    except Exception, e:
        shutil.rmtree(tmpd)
        print e
        return None
    
def create_certificate_file(manifest, certificate=None):
    """
    Create certificate file

    @param ovf: path to manifest file
    @type ovf: string
    @param manifest: path to certificate file
    @type manifest: string
    """
    class CreateCertOption:
        ovfFile = manifest
        certificateFile = certificate
    try:
        print 'calling makeCertificate'
        makeCertificate(CreateCertOption(), args = '')
        return True
    except Exception, e:
        print e
        return False
    
def create_manifest_file(ovf, manifest=None):
    """
    Create manifest file

    @param ovf: path to ovf file
    @type ovf: string
    @param manifest: path to manifest file
    @type manifest: string
    """

    class CreateManifestOptions:
        ovfFile = ovf
        manifestFile = manifest

    try:
        print 'calling makeManifest'
        makeManifest(CreateManifestOptions(), args = '')
        return True
    except Exception, e:
        print e
        return False

def validate_ova(ovf, manifest=None, certificate=None, install_loc=None):
    """
    Validate an appliance pacakge

    @param ovf: path to ova file
    @type ovf: string
    @param manifest: path to manifest file
    @type manifest: string
    @param certificate: path to certificate file
    @type certificate: string
    @param install_loc: path to destination
    @type install_loc: string
    """
    class CreateValidateOption:
        ovfFile = ovf
        manifestFile = manifest
        certFile = certificate
        installLoc = install_loc

    try :
        print 'calling validateAppliance '
        rc = validateAppliance(CreateValidateOption(), args = '' )

        return rc

    except Exception, e:
        print e
        return False

def pack_ova(ovf, outfile=None, manifest=None, 
                 certificate=None, nomanifest=False, nocertificate=True):
    """
    Package an appliance into an archive file using tar

    @param ovf: path to ovf file
    @type ovf: string
    @param manifest: path to manifest file
    @type manifest: string
    @param certificate: path to certificate file
    @type certificate: string
    @param nomanifest: Do not store manifest file in the archive
    @type nomanifest: False
    @param nocertificate: Do not store certificate file in the archive
    @type nocertificate: False
    """
    class CreatePackovaOption:
        ovfFile = ovf
        output = outfile
        manifestFile = manifest
        certificateFile = certificate
        noManifest = nomanifest
        noCertificate = nocertificate
    try :
        packOva(CreatePackovaOption(),args = '' )
        return True
    except Exception, e:
        print e
        return False

def unpack_ova(ova_file, target_dir=None):
    """
    Unpackage an appliance from an archive file

    @param ova_file: path to ova file
    @type ova_file: string
    @param target_dir: directory to store the unpackaged file
    @type target_dir: string
    """
    class CreateUnpackovaOption:
        ovfFile = ova_file
        targetDir = target_dir

    try :
        unpackOva(CreateUnpackovaOption(), args = '')
        return True
    except Exception, e:
        print e
        return False

def create_xmconfig_file(ovf, cfg_file=None, virt_sys_type=None, 
                         env_dir=None, config_id=None):
    """
    Create the Xen xm interface config file

    @param ovf: path to ovf file
    @type ovf: string
    @param cfg_file: file name of the Xen configuration file to create
    @type cfg_file: string
    @param virt_sys_type: virtual system type
    @type virt_sys_type: string
    @param env_dir: path to environment files
    @type env_dir: string
    @param config_id: configuration to use
    @type config_id: string
    """
    class CreateXmconfigfileOption:
        ovfFile = ovf
        cfgFile = cfg_file
        virtPlatform = virt_sys_type
        envDir = env_dir
        configId = config_id
    try :
        createXmConfigFile(CreateXmconfigfileOption(), args = '')
        return True
    except Exception, e:
        print e
        return False

def create_guest_environment(ovf, xmlfile=None, ovfid=None, config=None, 
                             virtplatform=None,  pkgdirectory=None, 
                             noprompt=False, novalue=False):
    """
    Generate the guest environment
    @param ovf: path to ovf file
    @type ovf: string
    @param xmlfile: path to Environment xml file
    @type xmlfile: string
    @param ovfid: virtual system ID
    @type ovfid: string
    @param config: configuration to use
    @type config: string
    @param virtplatform: virtualization platform
    @type virtplatform: string
    @param pkgdirectory: directory containing the package
    @type pkgdirectory: string
    @param noprompt: do not prompt for values, take all defaults
    @type noprompt: bool
    @param novalue: with noprompt, values without defaults are listed 
                    but left unset
    @type novalue: bool
    """
    class CreateGuestEnvOption:
        ovfFile = ovf
        xmlFile = xmlfile
        ovfId = ovfid
        configuration = config
        virtPlatform = virtplatform
        pkgDirectory = pkgdirectory
        noPrompt = noprompt
        noValue = novalue
    try :
        guestEnvironment(CreateGuestEnvOption(), args = '')
        return True
    except Exception, e:
        print e
        return False

def create_guest_transport(ovf, form='iso', outfile=None):
    """
    Generate the guest environment
    @param ovf: path to ovf file
    @type ovf: string
    @param form: transport type
    @type form: string
    @param outfile: file name, all environment files will be put on a 
                    single iso with this name
    @type outfile: string
    """
    class CreateGuestTpOption:
        ovfFile = ovf
        format = form
        outFile = outfile
    try :
        envTransport(CreateGuestTpOption(), args = '')
        return True
    except Exception, e:
        print e
        return False

if __name__ == "__main__":
    pass
